/* Custom styles for bijx documentation */

/* Logo styling */
.sidebar-logo {
    max-height: 5rem;
    margin: 0rem auto;
    display: block;
}

.sidebar-logo-container {
    margin: 0;
    margin-top: 1rem;
}

.sidebar-brand {
    padding-bottom: 0;
    padding-top: 0;
}

/* Ensure logo scales properly on mobile */
@media (max-width: 768px) {
    .sidebar-logo {
        max-height: 4rem;
    }
}

/* Style for any bijx branding elements */
.bijx-brand {
    color: #112136;
    font-weight: bold;
}

.bijx-accent {
    color: #aa4400;
}
